@@ -6,7 +6,8 @@ import typing
 
 from .lazy import Lazy, resolve_lazy
 
-"Turning on this flag will add html attributes with information about the source of the generated html output"
+EXCEPTION_HANDLER_NAME = "_htmlgenerator_exception_handler"
+"Must be a function without arguments, will be called when an exception happens during rendering an element"
 
 # for integration with the django safe string objects, optional
 try:
@@ -23,9 +24,9 @@ class BaseElement(list):
     def __init__(self, *children):
         """Uses the given arguments to initialize the list which represents the child objects"""
         super().__init__(children)
-        from . import __DEBUG__
+        from . import DEBUG
 
-        if __DEBUG__:
+        if DEBUG:
             # This will add the source location of where this element has been instantiated as a data attributte
             # and a python attribute _src_location with (filename, linenumber, functionname) to this object
             for frame in inspect.stack():
@@ -38,30 +39,73 @@ class BaseElement(list):
             self._src_location = (frame.filename, frame.lineno, frame.function)
 
     def _try_render(
-        self, element: typing.Any, context: dict
+        self, element: typing.Any, context: dict, stringify: bool
     ) -> typing.Generator[str, None, None]:
-        """Renders an element as a generator which yields strings"""
+        """Renders an element as a generator which yields strings
+        The stringify parameter should normally be true in order return
+        escaped strings. In some circumstances if is however desirable to
+        get the actual value and not a string returned. For such cases
+        stringify can be set to ``False``. An example are HTML attribute values
+        which us a ``hg.If`` element and return ``True`` or ``False`` to dynamically
+        control the appearance of the attribute.
+        The render_children method will use a default of ``True`` for strinfigy.
+        That behaviour should only be overriden by elements which consciously want
+        to be able to return non-string objects during rendering.
+        """
         while isinstance(element, Lazy):
             element = element.resolve(context, self)
         if isinstance(element, BaseElement):
             yield from element.render(context)
         elif element is not None:
-            yield conditional_escape(element)
+            yield conditional_escape(element) if stringify else element
 
-    def render_children(self, context: dict) -> typing.Generator[str, None, None]:
+    def render_children(
+        self, context: dict, stringify: bool = True
+    ) -> typing.Generator[str, None, None]:
         """Renders all elements inside the list. Can be used by subclassing elements if they need to controll where child elements are rendered."""
         for element in self:
-            yield from self._try_render(element, context)
+            yield from self._try_render(element, context, stringify)
 
-    def render(self, context: dict) -> typing.Generator[str, None, None]:
+    def render(
+        self, context: dict, stringify: bool = True
+    ) -> typing.Generator[str, None, None]:
         """Renders this element and its children. Can be overwritten by subclassing elements."""
         try:
-            yield from self.render_children(context)
+            yield from self.render_children(context, stringify)
         except (Exception, RuntimeError) as e:
+            import sys
             import traceback
 
-            traceback.print_exc()
-            raise RenderException(self, e)
+            def default_handler(context, message):
+                traceback.print_exc()
+                print(message, file=sys.stderr)
+
+            last_obj = None
+            indent = 0
+            message = []
+            for i in traceback.StackSummary.extract(
+                traceback.walk_tb(sys.exc_info()[2]), capture_locals=True
+            ):
+                if (
+                    i.locals is not None
+                    and "self" in i.locals
+                    and i.locals["self"] != last_obj
+                ):
+                    message.append(" " * indent + str(i.locals["self"]))
+                    last_obj = i.locals["self"]
+                    indent += 2
+            message.append(" " * indent + str(e))
+            message = "\n".join(message)
+
+            context.get(EXCEPTION_HANDLER_NAME, default_handler)(context, message)
+
+            yield (
+                ""
+                + '<pre style="border: solid 1px red; color: red; padding: 1rem; background-color: #ffdddd">'
+                + f"    <code>~~~ Exception: {conditional_escape(e)} ~~~</code>"
+                + "</pre>"
+                + f'<script>alert("Error: {conditional_escape(e)}")</script>'
+            )
 
     """
     Tree functions
@@ -160,24 +204,6 @@ class BaseElement(list):
     def copy(self) -> BaseElement:
         return copy.deepcopy(self)
 
-    def __repr__(self):
-        try:
-            attrs = ", ".join(
-                [
-                    f"{i}: {getattr(self, i)}"
-                    for i in dir(self)
-                    if not callable(getattr(self, i)) and not i.startswith("__")
-                ]
-            )
-            return (
-                f"{self.__class__.__name__}("
-                + ", ".join([i for i in (attrs, f"{len(self)} children") if i])
-                + ")"
-            )
-        except (Exception, RuntimeError):
-            # sometimes our nice serialization fails and we reverte to the dead-safe method of just printing the type of the object
-            return str(type(self))
-
 
 class If(BaseElement):
     def __init__(
@@ -190,11 +216,13 @@ class If(BaseElement):
         super().__init__(true_child, false_child)
         self.condition = condition
 
-    def render(self, context: dict):
+    def render(self, context: dict, stringify=True):
+        """The stringy argument can be set to False in order to get a python object
+        instead of a rendered string returned. This is usefull when evaluating"""
         if resolve_lazy(self.condition, context, self):
-            yield from self._try_render(self[0], context)
+            yield from self._try_render(self[0], context, stringify)
         elif len(self) > 1:
-            yield from self._try_render(self[1], context)
+            yield from self._try_render(self[1], context, stringify)
 
 
 class Iterator(BaseElement):
@@ -208,12 +236,12 @@ class Iterator(BaseElement):
         self.loopvariable = loopvariable
         super().__init__(content)
 
-    def render(self, context: dict):
+    def render(self, context: dict, stringify: bool = True):
         context = dict(context)
         for i, value in enumerate(resolve_lazy(self.iterator, context, self)):
             context[self.loopvariable] = value
             context[self.loopvariable + "_index"] = i
-            yield from self.render_children(context)
+            yield from self.render_children(context, stringify)
 
 
 class WithContext(BaseElement):
@@ -274,22 +302,6 @@ def render(root: BaseElement, basecontext: dict) -> str:
     return "".join(root.render(basecontext))
 
 
-class RenderException(Exception):
-    def __init__(self, elementtype: BaseElement, origin: BaseException):
-        self.trace: typing.Tuple[typing.Any, ...] = (
-            elementtype,
-            origin,
-        )
-        if isinstance(origin, RenderException):
-            self.trace = (elementtype,) + origin.trace
-
-    def __str__(self) -> str:
-        ret = []
-        for i, traceitem in enumerate(self.trace):
-            ret.append("    " * i + f"{traceitem}")
-        return "\n".join(ret)
-
-
 def print_logical_tree(root: BaseElement) -> None:
     from .htmltags import HTMLElement
 
